<!-- templates/index.html -->

<!DOCTYPE html>
<html>
<head>
    <title>Home</title>
    <style>
        .error-box {
            padding: 10px;
            margin: 15px 0;
            border: 1px solid red;
            background-color: #ffe6e6;
            color: red;
            border-radius: 5px;
        }
    </style>
</head>
<body>

    {% if error %}
        <div class="error-box">
            {{ error }}
        </div>
    {% endif %}

    <!-- Rest of your page content -->

</body>
</html>